use rocket::{form::Form, http::CookieJar, response::Redirect};
use rocket_dyn_templates::{context, Template};
use totp_rs::Secret;

use crate::{
	auth,
	db::{
		self,
		otp::METHOD_TOTP,
		schema::{DollTagsDb, User},
	},
	pages::CommonTemplateState,
	routes::{self, error_handlers::PageResult},
};

#[get("/settings/totp")]
pub async fn show_totp_enable_start(
	mut db: DollTagsDb,
	user: User,
	cookies: &CookieJar<'_>,
	meta: CommonTemplateState,
) -> PageResult {
	if db::otp::has_otp(&mut *db, &user.id, METHOD_TOTP).await? {
		return Ok(Redirect::to(uri!("/account", routes::account::settings::show_settings)).into());
	}
	let totp_secret = Secret::generate_secret();
	auth::otp::cache_secret(cookies, &totp_secret);

	let totp = auth::otp::make_totp(&user.id.to_string(), totp_secret.to_bytes()?)?;
	let totp_qrcode = totp.get_qr_base64()?;

	Ok(Template::render(
		"account/otp/start",
		context! {
			meta,
			totp_qrcode,
			secret: totp.get_secret_base32(),
		},
	)
	.into())
}

#[derive(FromForm)]
pub struct OtpEnableForm {
	#[field(validate = len(6..=6))]
	pub otp_code: String,
}

#[post("/settings/totp", data = "<form>")]
pub async fn handle_totp_enable_start(
	mut db: DollTagsDb,
	form: Form<OtpEnableForm>,
	user: User,
	cookies: &CookieJar<'_>,
	_meta: CommonTemplateState,
) -> PageResult {
	if db::otp::has_otp(&mut *db, &user.id, METHOD_TOTP).await? {
		return Ok(Redirect::to(uri!("/account", routes::account::settings::show_settings)).into());
	}
	let secret = match auth::otp::get_secret(cookies) {
		Some(v) => v,
		None => return Ok(Redirect::to(uri!("/account", show_totp_enable_start)).into()),
	};

	let totp = auth::otp::make_totp(&user.id.to_string(), secret.to_bytes()?)?;

	if !totp.check_current(&form.otp_code)? {
		Err(String::from(
			"woof TODO need to impl. a user error return w/ the same secret",
		))?;
	}

	todo!("meow")
}
